﻿using Discord;
using Discord.Commands;
using Discord.WebSocket;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace StatBot
{
    public class CommandsModule : ModuleBase<SocketCommandContext>
    {
        // Dependency Injection will fill this value in for us
        //public PictureService PictureService { get; set; }

        [Command("ping")]
        [Alias("test", "hello")]
        public Task PingAsync()
            => ReplyAsync("Bot works fine!");

        /*[Command("cat")]
        public async Task CatAsync()
        {
            // Get a stream containing an image of a cat
            var stream = await PictureService.GetCatPictureAsync();
            // Streams must be seeked to their beginning before being uploaded!
            stream.Seek(0, SeekOrigin.Begin);
            await Context.Channel.SendFileAsync(stream, "cat.png");
        }*/

        // Get info on a user, or the user who invoked the command if one is not specified
        /*[Command("userinfo")]
        public async Task UserInfoAsync(IUser user = null)
        {
            user = user ?? Context.User;

            await ReplyAsync(user.ToString());
        }

        // Ban a user
        [Command("ban")]
        [RequireContext(ContextType.Guild)]
        // make sure the user invoking the command can ban
        [RequireUserPermission(GuildPermission.BanMembers)]
        // make sure the bot itself can ban
        [RequireBotPermission(GuildPermission.BanMembers)]
        public async Task BanUserAsync(IGuildUser user, [Remainder] string reason = null)
        {
            await user.Guild.AddBanAsync(user, reason: reason);
            await ReplyAsync("ok!");
        }

        // [Remainder] takes the rest of the command's arguments as one argument, rather than splitting every space
        [Command("echo")]
        public Task EchoAsync([Remainder] string text)
            // Insert a ZWSP before the text to prevent triggering other bots!
            => ReplyAsync('\u200B' + text);

        // 'params' will parse space-separated elements into a list
        [Command("list")]
        public Task ListAsync(params string[] objects)
            => ReplyAsync("You listed: " + string.Join("; ", objects));*/

        /*[Command("guild_only")]
        [RequireContext(ContextType.Guild, ErrorMessage = "Sorry, this command must be ran from within a server, not a DM!")]
        public Task GuildOnlyCommand()
            => ReplyAsync("Nothing to see here!");*/

        #region Soldier info

        [Command("fName")]
        [Alias("findName", "searchName")]
        public async Task FindName([Remainder] string text)
        {
            var embed = new EmbedBuilder();

            if (Context.Guild == null)
            {
                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            if (!IsHavePermission((SocketGuildUser)Context.User, search.RolesHasPermission_SoldierInfo))
            {
                embed.WithDescription("Недостаточно прав для выполнения команды.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
            }
            else
            {
                string searchResult = search.GetSoldierInfoByName(text.Trim());

                embed.WithDescription(searchResult)
                .WithColor(new Color(0, 255, 0));
                await ReplyAsync(embed: embed.Build());
            }

        }

        [Command("fGuid")]
        [Alias("findGuid", "searchGuid")]
        public async Task FindGuid([Remainder] string text)
        {
            var embed = new EmbedBuilder();

            if (Context.Guild == null)
            {
                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            if (!IsHavePermission((SocketGuildUser)Context.User, search.RolesHasPermission_SoldierInfo))
            {
                embed.WithDescription("Недостаточно прав для выполнения команды.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
            }
            else
            {
                string searchResult = search.GetSoldierInfoByEAGuid(text.Trim());

                embed.WithDescription(searchResult)
                .WithColor(new Color(0, 255, 0));
                await ReplyAsync(embed: embed.Build());
            }
        }

        [Command("IpHistory")]
        public async Task ShowIpHistory([Remainder] string text)
        {
            var embed = new EmbedBuilder();

            if (Context.Guild == null)
            {
                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            if (!IsHavePermission((SocketGuildUser)Context.User, search.RolesHasPermission_SoldierInfo))
            {
                embed.WithDescription("Недостаточно прав для выполнения команды.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
            }
            else
            {
                string searchResult = search.GetSoldierIpHistoryByName(text.Trim());

                if (searchResult.Length > 2000)
                {
                    string result = string.Empty;

                    foreach (string item in searchResult.Split('\r'))
                    {
                        string line = item.Trim('\n');
                        if ((result.Length + line.Length + 2) > 2000)
                        {
                            embed.WithDescription(result)
                            .WithColor(new Color(0, 255, 0));
                            await ReplyAsync(embed: embed.Build());
                            result = line;
                        }
                        else
                            result += result.Length > 0 ? "\r\n" + line : line;
                    }

                    embed.WithDescription(result)
                    .WithColor(new Color(0, 255, 0));
                    await ReplyAsync(embed: embed.Build());
                }
                else
                    await ReplyAsync(
                        embed: embed.WithDescription(searchResult)
                        .WithColor(new Color(0, 255, 0)).Build()
                        );
            }

        }

        [Command("NameHistory")]
        public async Task ShowNameHistory([Remainder] string text)
        {
            var embed = new EmbedBuilder();

            if (Context.Guild == null)
            {
                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            if (!IsHavePermission((SocketGuildUser)Context.User, search.RolesHasPermission_SoldierInfo))
            {
                embed.WithDescription("Недостаточно прав для выполнения команды.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
            }
            else
            {
                string searchResult = search.GetSoldierNameHistoryByName(text.Trim());

                if (searchResult.Length > 2000)
                {
                    string result = string.Empty;

                    foreach (string item in searchResult.Split('\r'))
                    {
                        string line = item.Trim('\n');
                        if ((result.Length + line.Length + 2) > 2000)
                        {
                            embed.WithDescription(result)
                            .WithColor(new Color(0, 255, 0));
                            await ReplyAsync(embed: embed.Build());
                            result = line;
                        }
                        else
                            result += result.Length > 0 ? "\r\n" + line : line;
                    }

                    embed.WithDescription(result)
                    .WithColor(new Color(0, 255, 0));
                    await ReplyAsync(embed: embed.Build());
                }
                else
                    await ReplyAsync(
                        embed: embed.WithDescription(searchResult)
                        .WithColor(new Color(0, 255, 0)).Build()
                        );
            }
        }

        #endregion

        private bool IsHavePermission(SocketGuildUser user, List<string> commandRoles)
        {

            bool result = false;
            if (commandRoles == null || commandRoles.Count == 0)
                return result;
            foreach (SocketRole role in user.Roles)
            {
                result = commandRoles.Contains(role.Name);
                if (result)
                    break;
            }
            return result;
        }

        #region Statistic

        [Command("top10")]
        [Alias("top", "topt")]
        public async Task GetTop10()
        {
            EmbedBuilder embed;

            if (Context.Guild == null)
            {
                embed = new EmbedBuilder();

                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            {
                embed = search.GetTop10(search.serverIndex);
                await ReplyAsync(embed: embed.Build());
            }
        }

        [Command("stats")]
        public async Task GetPlayerStats([Remainder] string text)
        {
            EmbedBuilder embed;

            if (Context.Guild == null)
            {
                embed = new EmbedBuilder();

                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            if (!IsHavePermission((SocketGuildUser)Context.User, search.RolesHasPermission_SoldierStats))
            {
                embed = new EmbedBuilder();

                embed.WithDescription("Нет прав для просмотра статистики других игроков.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());

            }
            else
            {
                embed = search.GetStats(text.Trim(), search.serverIndex);
                await ReplyAsync(embed: embed.Build());
            }
        }

        [Command("stats")]
        public async Task GetPlayerStats()
        {
            EmbedBuilder embed;

            if (Context.Guild == null)
            {
                embed = new EmbedBuilder();

                embed.WithDescription("Выполнение команды в личных сообщениях запрещено.")
                .WithColor(new Color(255, 0, 0));
                await ReplyAsync(embed: embed.Build());
                return;
            }

            SearchService search = new SearchService();

            embed = search.GetStats(Context.User.Username, search.serverIndex);
            await ReplyAsync(embed: embed.Build());
        }

        #endregion

    }
}
