﻿using Discord;
using Discord.Commands;
using Discord.Net.Providers.WS4Net;
using Discord.WebSocket;
using Microsoft.Extensions.DependencyInjection;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Http;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;

namespace StatBot
{
    class Program
    {
        private static string botToken = string.Empty;
        private static DiscordSocketClient client = null;

        static void Main(string[] args)
        {
            Program program = new Program();
            Task task = program.MainAsync();
            TaskAwaiter awaiter = task.GetAwaiter();
            awaiter.GetResult();
            ConsoleKeyInfo key = new ConsoleKeyInfo();
            while (key.Key != ConsoleKey.Escape)
            {
                key = Console.ReadKey();
            }
            if (client != null && (client.ConnectionState == Discord.ConnectionState.Connected || client.ConnectionState == Discord.ConnectionState.Connecting))
            {
                program.StopClient();
                client = null;
            }
#if DEBUG
            Console.ReadKey();
#endif
        }

        public async Task StopClient()
        {
            await client.StopAsync();
        }

        public async Task MainAsync()
        {
            // You should dispose a service provider created using ASP.NET
            // when you are finished using it, at the end of your app's lifetime.
            // If you use another dependency injection framework, you should inspect
            // its documentation for the best way to do this.
            using (var services = ConfigureServices())
            {
                if (!GetSettings(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "bot.json")))
                {
                    Console.WriteLine("Error: Ошибка чтения файла конфигурации");
                    Console.ReadLine();
                    return;
                }

                client = services.GetRequiredService<DiscordSocketClient>();

                client.Log += LogAsync;
                services.GetRequiredService<CommandService>().Log += LogAsync;

                // Tokens should be considered secret data, and never hard-coded.
                //await client.LoginAsync(TokenType.Bot, Environment.GetEnvironmentVariable("token"));
                await client.LoginAsync(TokenType.Bot, botToken);
                await client.StartAsync();

                await services.GetRequiredService<CommandHandlingService>().InitializeAsync();

                //await Task.Delay(-1);
            }

        }

        private bool GetSettings(string configFile)
        {
            Dictionary<string, string> config;

            if (File.Exists(configFile))
            {
                try
                {
                    string json = File.ReadAllText(configFile);
                    var data = JsonConvert.DeserializeObject<dynamic>(json);
                    config = data.ToObject<Dictionary<string, string>>();
                    bool result = config != null && config.Count > 0;
                    if (result)
                    {
                        if (config.ContainsKey("botToken"))
                            botToken = config["botToken"];
                        else
                            result = false;
                    }
                    return result;
                }
                catch (Exception e)
                {
                    Console.WriteLine(string.Format("[{0} at SearchService] Error: {1}.", DateTime.Now.ToString("DD.MM.YYYY HH:mm:ss"), e.Message));
                    return false;
                }
            }
            else
                return false;
        }

        private ServiceProvider ConfigureServices()
        {
            DiscordSocketClient client = new DiscordSocketClient(new DiscordSocketConfig
            {
#if DEBUG
                LogLevel = LogSeverity.Debug,
#else
                LogLevel = LogSeverity.Error,
#endif
                WebSocketProvider = WS4NetProvider.Instance
            });

            CommandService service = new CommandService(new CommandServiceConfig
            {
                CaseSensitiveCommands = false,
                DefaultRunMode = RunMode.Async,
#if DEBUG
                LogLevel = LogSeverity.Debug
#else
                LogLevel = LogSeverity.Error
#endif
            });
            return new ServiceCollection()
                //.AddSingleton<SearchService>()
                .AddSingleton<DiscordSocketClient>(client)
                .AddSingleton<CommandService>(service)
                .AddSingleton<CommandHandlingService>()
                .AddSingleton<HttpClient>()
                //.AddSingleton<PictureService>()
                .BuildServiceProvider();
        }

        private Task LogAsync(LogMessage Message)
        {
            Console.WriteLine($"[{DateTime.Now} at {Message.Source}] {Message.Message}");
            return Task.CompletedTask;
        }


    }
}
