﻿using Discord;
using MySql.Data.MySqlClient;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace StatBot
{
    public class SearchService
    {
        //Database Connection Variables
        private string m_strHost = "";
        private string m_strDBPort = "";
        private string m_strDatabase = "";
        private string m_strUserName = "";
        private string m_strPassword = "";
        private string tableSuffix = "";

        private bool activeConnection = false;

        //Tablenames
        private string tbl_playerdata;
        private string tbl_playerstats;
        private string tbl_weaponstats;
        private string tbl_dogtags;
        private string tbl_mapstats;
        private string tbl_chatlog;
        private string tbl_bfbcs;
        private string tbl_awards;
        private string tbl_server;
        private string tbl_server_player;
        private string tbl_server_stats;
        private string tbl_playerrank;
        private string tbl_sessions;
        private string tbl_currentplayers;
        private string tbl_weapons;
        private string tbl_weapons_stats;
        private string tbl_games;
        private string tbl_teamscores;
        private string tbl_ip_history;
        private string tbl_name_history;

        private readonly object ConnectionStringBuilderlock;
        //private readonly object sqlquerylock;

        //private MySqlConnection MySqlCon; //Select Querys 1

        MySqlConnectionStringBuilder myCSB = new MySqlConnectionStringBuilder();

        public List<string> RolesHasPermission_SoldierInfo = null;
        public List<string> RolesHasPermission_SoldierStats = null;
        public int serverIndex = 1;

        public SearchService()
        {
            this.ConnectionStringBuilderlock = new object();
            prepareTablenames();

            if (!GetSettings(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "bot.json")))
            {
                DebugInfo("Error", "Ошибка чтения файла конфигурации");
                return;
            }

            if ((m_strHost != null) || (m_strDatabase != null) || (m_strDBPort != null) || (m_strUserName != null) || (m_strPassword != null))
            {
                try
                {
                    string query = "SELECT `ServerID` from " + this.tbl_server + " LIMIT 1";
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null)
                        {
                            activeConnection = true;
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                }
            }
        }

        private bool GetSettings(string configFile)
        {
            Dictionary<string, string> config;

            if (File.Exists(configFile))
            {
                try
                {
                    string json = File.ReadAllText(configFile);
                    var data = JsonConvert.DeserializeObject<dynamic>(json);
                    config = data.ToObject<Dictionary<string, string>>();
                    bool result = config != null && config.Count > 0;
                    if (result)
                    {
                        if (config.ContainsKey("m_strHost"))
                            m_strHost = config["m_strHost"];
                        else
                            result = false;

                        if (config.ContainsKey("m_strDBPort"))
                            m_strDBPort = config["m_strDBPort"];
                        else
                            result = false;

                        if (config.ContainsKey("m_strDatabase"))
                            m_strDatabase = config["m_strDatabase"];
                        else
                            result = false;

                        if (config.ContainsKey("m_strUserName"))
                            m_strUserName = config["m_strUserName"];
                        else
                            result = false;

                        if (config.ContainsKey("m_strPassword"))
                            m_strPassword = config["m_strPassword"];
                        else
                            result = false;

                        if (config.ContainsKey("tableSuffix"))
                            tableSuffix = config["tableSuffix"];
                        else
                            result = false;

                        if (config.ContainsKey("RolesHasPermission_SoldierInfo"))
                            RolesHasPermission_SoldierInfo = new List<string>(config["RolesHasPermission_SoldierInfo"].Split('|'));

                        if (config.ContainsKey("RolesHasPermission_SoldierStats"))
                            RolesHasPermission_SoldierStats = new List<string>(config["RolesHasPermission_SoldierStats"].Split('|'));

                        if (config.ContainsKey("serverIndex") && !Int32.TryParse(config["serverIndex"], out serverIndex))
                            serverIndex = 1;
                        /*
                        if (config.ContainsKey("tableSuffix"))
                            tableSuffix = config["tableSuffix"];
                        /*
                        if (config.ContainsKey("tableSuffix"))
                            tableSuffix = config["tableSuffix"];*/
                    }
                    return result;
                }
                catch (Exception e)
                {
                    DebugInfo("Error", e.Message);
                    return false;
                }
            }
            else
                return false;
        }

        private void DebugInfo(string type, string message)
        {
            Console.WriteLine(string.Format("[{0} at SearchService] {1}: {2}.", DateTime.Now.ToString("DD.MM.YYYY HH:mm:ss"), type, message));
        }

        public void DisplayMySqlErrorCollection(MySqlException myException)
        {
            if (myException != null)
            {
                this.DebugInfo("SQL Error Message", myException.Message);
                this.DebugInfo("SQL Error Native", myException.ErrorCode.ToString());
                this.DebugInfo("SQL Error Source", myException.Source);
                this.DebugInfo("SQL Error StackTrace", myException.StackTrace);
                if (myException.InnerException != null)
                    this.DebugInfo("SQL Error InnerException", myException.InnerException.ToString());
            }
        }

        private string DBConnectionStringBuilder()
        {
            string conString = String.Empty;
            lock (this.ConnectionStringBuilderlock)
            {
                uint uintport = 3306;
                uint.TryParse(m_strDBPort, out uintport);
                myCSB.Port = uintport;
                myCSB.Server = m_strHost;
                myCSB.UserID = m_strUserName;
                myCSB.Password = m_strPassword;
                myCSB.Database = m_strDatabase;
                //Connection Pool
                /*if (this.m_connectionPooling == enumBoolOnOff.On)
                {
                    myCSB.Pooling = true;
                    myCSB.MinimumPoolSize = Convert.ToUInt32(this.m_minPoolSize);
                    myCSB.MaximumPoolSize = Convert.ToUInt32(this.m_maxPoolSize);
                    myCSB.ConnectionLifeTime = 600;
                }
                else*/
                {
                    myCSB.Pooling = false;
                }
                //Compression
                /*if (this.m_Connectioncompression == enumBoolOnOff.On)
                {
                    myCSB.UseCompression = true;
                }
                else*/
                {
                    myCSB.UseCompression = false;
                }
                myCSB.AllowUserVariables = true;
                myCSB.DefaultCommandTimeout = 3600;
                myCSB.ConnectionTimeout = 50;
                conString = myCSB.ConnectionString;
            }
            return conString;
        }

        private void prepareTablenames()
        {
            this.tbl_playerdata = m_strDatabase + ".tbl_playerdata" + this.tableSuffix;
            this.tbl_playerstats = m_strDatabase + ".tbl_playerstats" + this.tableSuffix;
            this.tbl_weaponstats = m_strDatabase + ".tbl_weaponstats" + this.tableSuffix;
            this.tbl_dogtags = m_strDatabase + ".tbl_dogtags" + this.tableSuffix;
            this.tbl_mapstats = m_strDatabase + ".tbl_mapstats" + this.tableSuffix;
            this.tbl_chatlog = m_strDatabase + ".tbl_chatlog" + this.tableSuffix;
            this.tbl_bfbcs = m_strDatabase + ".tbl_bfbcs" + this.tableSuffix;
            this.tbl_awards = m_strDatabase + ".tbl_awards" + this.tableSuffix;
            this.tbl_server = m_strDatabase + ".tbl_server" + this.tableSuffix;
            this.tbl_server_player = m_strDatabase + ".tbl_server_player" + this.tableSuffix;
            this.tbl_server_stats = m_strDatabase + ".tbl_server_stats" + this.tableSuffix;
            this.tbl_playerrank = m_strDatabase + ".tbl_playerrank" + this.tableSuffix;
            this.tbl_sessions = m_strDatabase + ".tbl_sessions" + this.tableSuffix;
            this.tbl_currentplayers = m_strDatabase + ".tbl_currentplayers" + this.tableSuffix;
            this.tbl_weapons = m_strDatabase + ".tbl_weapons" + this.tableSuffix;
            this.tbl_weapons_stats = m_strDatabase + ".tbl_weapons_stats" + this.tableSuffix;
            this.tbl_games = m_strDatabase + ".tbl_games" + this.tableSuffix;
            this.tbl_teamscores = m_strDatabase + ".tbl_teamscores" + this.tableSuffix;
            this.tbl_ip_history = m_strDatabase + ".ip_history" + this.tableSuffix;
            this.tbl_name_history = m_strDatabase + ".soldiername_history" + this.tableSuffix;
        }

        private DataTable SQLquery(MySqlCommand selectQuery)
        {
            DataTable MyDataTable = new DataTable();
            try
            {
                //this.tablebuilder();
                //this.DebugInfo("Trace", "SQLquery: " + selectQuery);                
                if (selectQuery == null)
                {
                    this.DebugInfo("Warning", "SQLquery: selectQuery is null");
                    return MyDataTable;
                }
                else if (selectQuery.CommandText.Equals(String.Empty) == true)
                {
                    this.DebugInfo("Warning", "SQLquery: CommandText is empty");
                    return MyDataTable;
                }

                //if (this.m_highPerformanceConnectionMode == enumBoolOnOff.On)
                {
                    try
                    {
                        using (MySqlConnection Connection = new MySqlConnection(this.DBConnectionStringBuilder()))
                        {
                            selectQuery.Connection = Connection;
                            using (MySqlDataAdapter MyAdapter = new MySqlDataAdapter(selectQuery))
                            {
                                if (MyAdapter != null)
                                {
                                    MyAdapter.Fill(MyDataTable);
                                }
                                else
                                {
                                    this.DebugInfo("Warning", "SQLquery: MyAdapter is null");
                                }
                            }
                            Connection.Close();
                        }
                    }
                    catch (MySqlException me)
                    {
                        this.DebugInfo("Error", "SQLQuery:");
                        this.DisplayMySqlErrorCollection(me);
                    }
                    catch (Exception c)
                    {
                        this.DebugInfo("Error", "SQLQuery:" + c);
                    }
                }
            }
            catch (Exception c)
            {
                this.DebugInfo("Error", "SQLQuery OuterException:" + c);
            }
            return MyDataTable;
        }

        #region Soldier info

        public string GetSoldierInfoByName(string soldierName)
        {
            StringBuilder result = new StringBuilder();
            if (activeConnection)
            {
                try
                {
                    bool like = soldierName.StartsWith("*") || soldierName.EndsWith("*");
                    StringBuilder str = new StringBuilder();
                    str.AppendLine(string.Format("SELECT pd.Soldiername, pd.EAGUID, pd.IP_Address, ps.LastSeenOnServer FROM {0} ps", this.tbl_playerstats));
                    str.AppendLine(string.Format("left outer join {0} sp on sp.StatsID = ps.StatsID and sp.ServerID = 1", this.tbl_server_player));
                    str.AppendLine(string.Format("left outer join {0} pd on pd.PlayerID = sp.PlayerID", this.tbl_playerdata));
                    str.AppendLine(string.Format("where lower(pd.SoldierName) {0} lower(@p_name);", like ? "like" : "="));

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@p_name", like ? soldierName.Replace('*', '%') : soldierName);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count > 0)
                        {
                            if (resultTable.Rows.Count > 1)
                            {
                                int i = 0;
                                result.AppendLine(string.Format(@"Поиск игрока по нику '{0}', найдено совпадений: {1}", soldierName.Replace("*", ""), resultTable.Rows.Count));
                                result.AppendLine();
                                if (resultTable.Columns.Contains("SoldierName"))
                                {
                                    foreach (DataRow row in resultTable.Rows)
                                    {
                                        result.AppendFormat("{0}, ", row["SoldierName"]);
                                        i++;
                                        if (i > 30)
                                        {
                                            result.AppendFormat("всего совпадений: {0}.", resultTable.Rows.Count);
                                            break;
                                        };
                                    }
                                }
                                return result.ToString().Trim().TrimEnd(',');
                            }
                            else
                            {
                                DataRow row = resultTable.Rows[0];
                                result.AppendLine(string.Format("Игрок: {0}", row["SoldierName"]));
                                result.AppendLine(string.Format("EAGUID: {0}", row["EAGUID"]));
                                result.AppendLine(string.Format("IP: {0}", row["IP_Address"]));
                                result.AppendLine(string.Format("Последний вход: {0}", row["LastSeenOnServer"]));
                                return result.ToString();
                            }

                        }
                        else
                        {
                            return string.Format("Игрок с именем '{0}' не найден.\r\nИспользуйте '*' для поиска по частичному совпадению имени.", soldierName.Replace("*", ""));
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    return "Возникла непредвиденная ошибка при обращении к БД.";
                }

            }
            else
            {
                return "Ошибка: Нет соединения с базой данных.";
            }
        }

        public string GetSoldierInfoByEAGuid(string EAGuid)
        {
            StringBuilder result = new StringBuilder();
            if (activeConnection)
            {
                try
                {
                    StringBuilder str = new StringBuilder();
                    str.AppendLine(string.Format("SELECT pd.Soldiername, pd.EAGUID, pd.IP_Address, ps.LastSeenOnServer FROM {0} ps", this.tbl_playerstats));
                    str.AppendLine(string.Format("left outer join {0} sp on sp.StatsID = ps.StatsID and sp.ServerID = 1", this.tbl_server_player));
                    str.AppendLine(string.Format("left outer join {0} pd on pd.PlayerID = sp.PlayerID", this.tbl_playerdata));
                    str.AppendLine("where lower(pd.EAGUID) = lower(@p_guid);");

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@p_guid", EAGuid);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count == 1)
                        {
                            DataRow row = resultTable.Rows[0];
                            result.AppendLine(string.Format("Игрок: {0}", row["SoldierName"]));
                            result.AppendLine(string.Format("EAGUID: {0}", row["EAGUID"]));
                            result.AppendLine(string.Format("IP: {0}", row["IP_Address"]));
                            result.AppendLine(string.Format("Последний вход: {0}", row["LastSeenOnServer"]));
                            return result.ToString();
                        }
                        else
                        {
                            return string.Format(@"Игрок с EAGUID '{0}' не найден.", EAGuid);
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    return "Возникла непредвиденная ошибка при обращении к БД.";
                }

            }
            else
            {
                return "Ошибка: Нет соединения с базой данных.";
            }
        }

        public string GetSoldierIpHistoryByName(string soldierName)
        {
            StringBuilder result = new StringBuilder();
            if (activeConnection)
            {
                try
                {
                    StringBuilder str = new StringBuilder();
                    str.AppendLine(string.Format("SELECT pd.IP_Address as Current_IP, ip.IP_Address, ip.RecStamp FROM {0} pd", this.tbl_playerdata));
                    str.AppendLine(string.Format("left outer join {0} ip on pd.PlayerID = ip.PlayerID", this.tbl_ip_history));
                    str.AppendLine("where lower(pd.SoldierName) = lower(@p_name) order by ip.RecStamp desc;");

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@p_name", soldierName);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count > 0)
                        {
                            result.AppendLine(string.Format(@"Поиск истории IP игрока по нику '{0}', найдено записей: {1}", soldierName.Replace("*", ""), resultTable.Rows.Count));
                            result.AppendLine(string.Format("Текущий IP: {0}", resultTable.Rows[0]["Current_IP"]));
                            result.AppendLine();
                            result.AppendLine("Журнал изменений:");

                            foreach (DataRow row in resultTable.Rows)
                            {
                                if (row["RecStamp"] != DBNull.Value)
                                    result.AppendLine(string.Format("{0}:   {1}", Convert.ToDateTime(row["RecStamp"]).ToString("dd.MM.yyyy HH:mm:ss"), row["IP_Address"]));
                            }

                            return result.ToString();
                        }
                        else
                        {
                            return string.Format(@"История IP игрока с именем '{0}' пуста.", soldierName.Replace("*", ""));
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    return "Возникла непредвиденная ошибка при обращении к БД.";
                }

            }
            else
            {
                return "Ошибка: Нет соединения с базой данных.";
            }
        }

        public string GetSoldierNameHistoryByName(string soldierName)
        {
            StringBuilder result = new StringBuilder();
            if (activeConnection)
            {
                try
                {
                    StringBuilder str = new StringBuilder();
                    str.AppendLine(string.Format("SELECT nm.Old_SoldierName, nm.New_SoldierName, nm.RecStamp FROM {0} pd", this.tbl_playerdata));
                    str.AppendLine(string.Format("left outer join {0} nm on pd.PlayerID = nm.PlayerID", this.tbl_name_history));
                    str.AppendLine("where lower(pd.SoldierName) = lower(@p_name) order by nm.RecStamp desc;");

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@p_name", soldierName);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count > 0 && resultTable.Rows[0]["RecStamp"] != DBNull.Value)
                        {
                            result.AppendLine(string.Format(@"Поиск истории имен игрока по нику '{0}', найдено записей: {1}", soldierName.Replace("*", ""), resultTable.Rows.Count));
                            result.AppendLine();
                            result.AppendLine("Журнал изменений:");

                            foreach (DataRow row in resultTable.Rows)
                            {
                                if (row["RecStamp"] != DBNull.Value)
                                    result.AppendLine(string.Format("{0}:   {1} ==> {2}", Convert.ToDateTime(row["RecStamp"]).ToString("dd.MM.yyyy HH:mm:ss"),
                                        row["Old_SoldierName"], row["New_SoldierName"]));
                            }

                            return result.ToString();
                        }
                        else
                        {
                            if (resultTable.Rows != null && resultTable.Rows.Count > 0)
                                return string.Format(@"История имен игрока с ником '{0}' пуста.", soldierName.Replace("*", ""));
                            else
                                return string.Format(@"Игрок с ником '{0}' не найден.", soldierName.Replace("*", ""));
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    return "Возникла непредвиденная ошибка при обращении к БД.";
                }

            }
            else
            {
                return "Ошибка: Нет соединения с базой данных.";
            }
        }

        #endregion

        #region Statistic

        public EmbedBuilder GetTop10(int ServerID = 1)
        {
            EmbedBuilder result = new EmbedBuilder();
            result.WithColor(new Color(255, 0, 0));

            if (activeConnection)
            {
                try
                {
                    StringBuilder str = new StringBuilder();

                    str.AppendLine(string.Format("SELECT tpd.SoldierName, tps.Score, tps.Kills, tps.Deaths, tps.Headshots FROM {0} tps", this.tbl_playerstats));
                    str.AppendLine(string.Format("INNER JOIN {0} tsp ON tsp.StatsID = tps.StatsID", this.tbl_server_player));
                    str.AppendLine(string.Format("INNER JOIN {0} tpd ON tsp.PlayerID = tpd.PlayerID", this.tbl_playerdata));
                    str.AppendLine("WHERE tsp.ServerID = @ServerID AND tps.rankScore BETWEEN 1 AND 10 ORDER BY tps.rankScore ASC");

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@ServerID", ServerID);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count > 0)
                        {
                            result.WithTitle("Top 10 игроков сервера:");

                            //Top 10 Header
                            StringBuilder Top10Row = new StringBuilder();
                            int rank = 0;
                            double kdr1;
                            double khr;

                            foreach (DataRow row in resultTable.Rows)
                            {
                                Top10Row.AppendLine("Счет: %playerScore%;  %playerKills% Kills,  %playerDeaths% Deaths,  %playerHeadshots% Headshots,  KDR: %playerKDR%");

                                if (Convert.ToDouble(row["Deaths"]) != 0)
                                {
                                    kdr1 = Convert.ToDouble(row["Kills"]) / Convert.ToDouble(row["Deaths"]);
                                    Top10Row.Replace("%playerKDR%", Math.Round(kdr1, 2).ToString());
                                }
                                else
                                {
                                    Top10Row.Replace("%playerKDR%", Convert.ToDouble(row["Kills"]).ToString());
                                }

                                if (Convert.ToDouble(row["Headshots"]) != 0)
                                {
                                    khr = Convert.ToDouble(row["Kills"]) / Convert.ToDouble(row["Headshots"]);
                                    khr = Math.Round(khr, 4);
                                    khr = khr * 100;
                                    Top10Row.Replace("%playerKHR%", khr.ToString());
                                }
                                else
                                {
                                    khr = 0;
                                }
                                rank++;

                                Top10Row.Replace("%playerScore%", row["Score"].ToString()).Replace("%playerKills%", row["Kills"].ToString());
                                Top10Row.Replace("%playerDeaths%", row["Deaths"].ToString()).Replace("%playerHeadshots%", row["Headshots"].ToString());

                                result.AddField($"{rank.ToString("00")}. {row["SoldierName"].ToString()}", Top10Row.ToString());

                                //result.Append(Top10Row.ToString());
                                Top10Row.Length = 0;
                            }
                            result.WithColor(new Color(10, 120, 100));
                            return result;
                        }
                        else
                        {
                            result.WithTitle("Top 10 игроков сервера:");
                            result.WithDescription("Непредвиденная ошибка получения статистики.");
                            return result;
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    result.WithTitle("Top 10 игроков сервера:");
                    result.WithDescription("Возникла непредвиденная ошибка при обращении к БД.");
                    return result;
                }

            }
            else
            {
                result.WithTitle("Top 10 игроков сервера:");
                result.WithDescription("Ошибка: Нет соединения с базой данных.");
                return result;
            }
        }

        public EmbedBuilder GetStats(string SoldierName, int ServerID = 1)
        {
            EmbedBuilder result = new EmbedBuilder();
            result.WithColor(new Color(255, 0, 0));

            if (activeConnection)
            {
                try
                {
                    StringBuilder str = new StringBuilder();

                    str.AppendLine("SELECT tpd.SoldierName AS SoldierName, tps.Score AS Score, tps.Kills AS Kills, tps.Deaths AS Deaths, tps.Suicide AS Suicide, tps.TKs AS TKs,");
                    str.AppendLine($"tps.rankScore AS rank, (SELECT tss.CountPlayers FROM {this.tbl_server_stats} tss WHERE ServerID = @ServerID ) AS allrank , tps.Playtime AS Playtime,");
                    str.AppendLine("tps.Headshots AS Headshots, tps.Rounds AS Rounds, tps.Killstreak AS Killstreak, tps.Deathstreak AS Deathstreak, tps.FirstSeenOnServer AS FirstSeenOnServer");
                    str.AppendLine($"FROM {this.tbl_playerdata} tpd INNER JOIN {this.tbl_server_player} tsp ON tsp.PlayerID = tpd.PlayerID");
                    str.AppendLine($"INNER JOIN {this.tbl_playerstats} tps ON tps.StatsID = tsp.StatsID");
                    str.AppendLine("WHERE  tsp.ServerID = @ServerID AND tpd.SoldierName = @SoldierName");

                    string query = str.ToString();
                    using (MySqlCommand command = new MySqlCommand(query))
                    {
                        command.Parameters.AddWithValue("@ServerID", ServerID);
                        command.Parameters.AddWithValue("@SoldierName", SoldierName);

                        DataTable resultTable = this.SQLquery(command);
                        if (resultTable.Rows != null && resultTable.Rows.Count > 0)
                        {
                            //result.WithTitle("Статистика");

                            //Top 10 Header
                            StringBuilder StatRow = new StringBuilder();

                            foreach (DataRow row in resultTable.Rows)
                            {
                                StatRow.AppendLine(@"Счет: %playerScore%,  %playerKills% Kills, %playerDeaths% Deaths, KDR: %playerKDR%, Headshots: %playerHeadshots%");
                                StatRow.AppendLine(@"Ранг: %playerRank% из %allRanks%; Первое посещение сервера: %FirstSeenOnServer%");
                                StatRow.AppendLine();

                                StatRow = StatRow
                                .Replace("%playerScore%", row["Score"].ToString())
                                .Replace("%playerKills%", row["Kills"].ToString())
                                .Replace("%playerDeaths%", row["Deaths"].ToString())
                                .Replace("%playerSuicide%", row["Suicide"].ToString())
                                .Replace("%playerTKs%", row["TKs"].ToString())
                                .Replace("%playerRank%", row["rank"].ToString())
                                .Replace("%allRanks%", row["allrank"].ToString())
                                .Replace("%playerHeadshots%", row["Headshots"].ToString())
                                .Replace("%rounds%", row["Rounds"].ToString())
                                .Replace("%killstreak%", row["Killstreak"].ToString())
                                .Replace("%deathstreak%", row["Deathstreak"].ToString());

                                //KDR
                                double kdr = 0;
                                if (Convert.ToInt32(row["Deaths"]) != 0)
                                {
                                    kdr = Convert.ToDouble(row["Kills"]) / Convert.ToDouble(row["Deaths"]);
                                    kdr = Math.Round(kdr, 2);
                                    StatRow = StatRow.Replace("%playerKDR%", kdr.ToString());
                                }
                                else
                                {
                                    kdr = Convert.ToDouble(row["Kills"]);
                                    StatRow = StatRow.Replace("%playerKDR%", kdr.ToString());
                                }

                                //Playtime
                                TimeSpan span = new TimeSpan(0, 0, Convert.ToInt32(row["Playtime"]));
                                StatRow = StatRow.Replace("%playerPlaytime%", span.ToString());

                                //SPM
                                double SPM;
                                if (Convert.ToDouble(row["Playtime"]) != 0)
                                {
                                    SPM = (Convert.ToDouble(row["Score"]) / (Convert.ToDouble(row["Playtime"]) / 60));
                                    SPM = Math.Round(SPM, 2);
                                    StatRow = StatRow.Replace("%SPM%", SPM.ToString());
                                }
                                else
                                {
                                    StatRow = StatRow.Replace("%SPM%", "0");
                                }

                                //FirstSeen
                                DateTime data;
                                if (row["FirstSeenOnServer"] != DBNull.Value)
                                {
                                    data = Convert.ToDateTime(row["FirstSeenOnServer"]);
                                    StatRow = StatRow.Replace("%FirstSeenOnServer%", data.ToString("dd.MM.yyyy HH:mm:ss"));
                                }
                                else
                                {
                                    StatRow = StatRow.Replace("%FirstSeenOnServer%", "");
                                }

                                result.AddField($"Статистика сервера для {row["SoldierName"].ToString()}:", StatRow.ToString());
                                StatRow.Length = 0;
                            }

                            result.WithColor(new Color(10, 120, 100));
                            return result;
                        }
                        else
                        {
                            result.WithTitle($"Статистика сервера для {SoldierName}:");
                            result.WithDescription("игрок не найден, дождитесь окончания раунда.");
                            return result;
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine($"{DateTime.Now} at SearchService] Initialization Error: {e.Message}");
                    //result.WithTitle("Top 10 игроков сервера:");
                    result.WithDescription("Возникла непредвиденная ошибка при обращении к БД.");
                    return result;
                }

            }
            else
            {
                //result.WithTitle("Top 10 игроков сервера:");
                result.WithDescription("Ошибка: Нет соединения с базой данных.");
                return result;
            }
        }


        #endregion
    }

}
